@@ -45,7 +45,7 @@
 #ifdef SDL_VIDEO_DRIVER_WINDOWS
 extern HWND SDL_HelperWindow;
 #else
-static const HWND SDL_HelperWindow = NULL;
+static HWND SDL_HelperWindow = NULL;
 #endif
 
 /* local variables */
@@ -373,6 +373,8 @@ DIEFFECT *CreateRumbleEffectData(Sint16 magnitude)
         FreeRumbleEffectData(effect);
         return NULL;
     }
+    effect->rgdwAxes[0] = DIJOFS_X;
+    effect->rgdwAxes[1] = DIJOFS_Y;
 
     effect->rglDirection = (LONG *)SDL_calloc(effect->cAxes, sizeof(LONG));
     if (!effect->rglDirection) {
@@ -762,13 +764,15 @@ int SDL_DINPUT_JoystickOpen(SDL_Joystick *joystick, JoyStick_DeviceData *joystic
 
     /* Acquire shared access. Exclusive access is required for forces,
      * though. */
+#ifndef SDL_VIDEO_DRIVER_WINDOWS
+    /* Create invisible window. */
+    SDL_HelperWindow = CreateWindowExA(0, "STATIC", "", WS_DISABLED,
+                                       0, 0, 0, 0, NULL, NULL, NULL, NULL);
+    ShowWindow(SDL_HelperWindow, SW_HIDE);
+#endif
     result =
         IDirectInputDevice8_SetCooperativeLevel(joystick->hwdata->InputDevice, SDL_HelperWindow,
-#ifdef SDL_VIDEO_DRIVER_WINDOWS
                                                 DISCL_EXCLUSIVE |
-#else
-                                                DISCL_NONEXCLUSIVE |
-#endif
                                                     DISCL_BACKGROUND);
     if (FAILED(result)) {
         return SetDIerror("IDirectInputDevice8::SetCooperativeLevel", result);
@@ -914,26 +918,29 @@ int SDL_DINPUT_JoystickRumble(SDL_Joystick *joystick, Uint16 low_frequency_rumbl
         return SDL_Unsupported();
     }
 
-    if (joystick->hwdata->ff_initialized) {
-        DIPERIODIC *periodic = ((DIPERIODIC *)joystick->hwdata->ffeffect->lpvTypeSpecificParams);
-        periodic->dwMagnitude = CONVERT_MAGNITUDE(magnitude);
-
-        result = IDirectInputEffect_SetParameters(joystick->hwdata->ffeffect_ref, joystick->hwdata->ffeffect, (DIEP_DURATION | DIEP_TYPESPECIFICPARAMS));
-        if (result == DIERR_INPUTLOST) {
-            result = IDirectInputDevice8_Acquire(joystick->hwdata->InputDevice);
-            if (SUCCEEDED(result)) {
-                result = IDirectInputEffect_SetParameters(joystick->hwdata->ffeffect_ref, joystick->hwdata->ffeffect, (DIEP_DURATION | DIEP_TYPESPECIFICPARAMS));
-            }
-        }
-        if (FAILED(result)) {
-            return SetDIerror("IDirectInputDevice8::SetParameters", result);
-        }
-    } else {
+    if (!joystick->hwdata->ff_initialized) {
         if (SDL_DINPUT_JoystickInitRumble(joystick, magnitude) < 0) {
             return -1;
         }
         joystick->hwdata->ff_initialized = SDL_TRUE;
     }
+    do {
+        DIPERIODIC *periodic = ((DIPERIODIC *)joystick->hwdata->ffeffect->lpvTypeSpecificParams);
+        periodic->dwMagnitude = CONVERT_MAGNITUDE(magnitude);
+        joystick->hwdata->ffeffect->rglDirection[0] = low_frequency_rumble;
+        joystick->hwdata->ffeffect->rglDirection[1] = high_frequency_rumble;
+
+        result = IDirectInputEffect_SetParameters(joystick->hwdata->ffeffect_ref, joystick->hwdata->ffeffect, (DIEP_DURATION | DIEP_DIRECTION | DIEP_TYPESPECIFICPARAMS));
+        if (result == DIERR_INPUTLOST) {
+            result = IDirectInputDevice8_Acquire(joystick->hwdata->InputDevice);
+            if (SUCCEEDED(result)) {
+                result = IDirectInputEffect_SetParameters(joystick->hwdata->ffeffect_ref, joystick->hwdata->ffeffect, (DIEP_DURATION | DIEP_DIRECTION | DIEP_TYPESPECIFICPARAMS));
+            }
+        }
+        if (FAILED(result)) {
+            return SetDIerror("IDirectInputDevice8::SetParameters", result);
+        }
+    } while (0);
 
     result = IDirectInputEffect_Start(joystick->hwdata->ffeffect_ref, 1, 0);
     if (result == DIERR_INPUTLOST || result == DIERR_NOTEXCLUSIVEACQUIRED) {
@@ -1151,6 +1158,12 @@ void SDL_DINPUT_JoystickClose(SDL_Joystick *joystick)
     IDirectInputDevice8_Unacquire(joystick->hwdata->InputDevice);
     IDirectInputDevice8_Release(joystick->hwdata->InputDevice);
     joystick->hwdata->ff_initialized = SDL_FALSE;
+#ifndef SDL_VIDEO_DRIVER_WINDOWS
+    if (SDL_HelperWindow) {
+        DestroyWindow(SDL_HelperWindow);
+        SDL_HelperWindow = NULL;
+    }
+#endif
 }
 
 void SDL_DINPUT_JoystickQuit(void)
